<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Block\Adminhtml\System\Config;

use Magento\Backend\Block\Template\Context;
use Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\View\Element\Html\Select;

/**
 * Class AbstractConfig
 * @package FiloBlu\MappConnector\Block\Adminhtml\System\Config
 */
class AbstractConfig extends AbstractFieldArray
{
    /**
     * @var string
     */
    protected $_template = 'FiloBlu_MappConnector::system/config/form/field/array.phtml';

    /**
     * @var string
     */
    protected $_id;

    /**
     * @var array
     */
    protected $_exclude = [];

    /**
     * @var string
     */
    protected $_type;

    /**
     * Custom Model
     */
    protected $_model;

    /**
     * @var array
     */
    protected $_additionalOptions = [];

    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * AbstractConfig constructor.
     * @param Context $context
     * @param ResourceConnection $resourceConnection
     * @param array $data
     */
    public function __construct(
        Context $context,
        ResourceConnection $resourceConnection,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * Prepare to render
     *
     * @return void
     * @throws LocalizedException
     */
    protected function _prepareToRender()
    {
        $customAttributes = $this->getAttributesToRender();
        array_unshift($customAttributes, ['value' => '', 'label' => __('-- Select an attribute --')]);

        $customAttributesBlock =
            $this->getLayout()
                ->createBlock(Select::class)
                ->setId($this->_getCellInputElementId('<%- _id %>', $this->_id))
                ->setName($this->_getCellInputElementName($this->_id))
                ->setClass('select')
                ->setOptions($customAttributes);

        $this->addColumn("{$this->_id}_magento_field", [
            'label' => __('Magento Field'),
            'renderer' => $customAttributesBlock
        ]);
        $this->addColumn("{$this->_id}_mapp_field", [
            'label' => __('MAPP Field'),
            'style' => 'width:220px'
        ]);

        $this->_addAfter = false;
        $this->_addButtonLabel = __('Add');
    }

    /**
     * Gets the attribute to render
     *
     * @return array
     */
    protected function getAttributesToRender()
    {
        $customOptions = [];

        if ($this->_type == 'eav') {
            $attributes = $this->_model->getAttributes();
            foreach ($attributes as $k => $v) {
                if (!in_array($k, $this->_exclude)) {
                    $frontendLabel = $v->getFrontendLabel();
                    if (!$frontendLabel) {
                        $frontendLabel = ucwords(str_replace('_', ' ', $k));
                    }
                    $customOptions[$k] = ['value' => $k, 'label' => "{$k} ({$frontendLabel})" ];
                }
            }
        } elseif ($this->_type == 'flat') {
            $connection = $this->resourceConnection->getConnection(ResourceConnection::DEFAULT_CONNECTION);
            $tableName = $this->_model->getCollection()->getMainTable();
            $attributes = $connection->describeTable($tableName);
            foreach ($attributes as $k => $v) {
                if (!in_array($k, $this->_exclude)) {
                    $frontendLabel = str_replace('_', ' ', ucwords($k));
                    $customOptions[$k] = ['value' => $k, 'label' =>  "{$k} ({$frontendLabel})" ];
                }
            }
        }

        if (count($this->_additionalOptions)) {
            foreach ($this->_additionalOptions as $option) {
                $customOptions[$option['value']] = $option;
            }
        }

        ksort($customOptions);

        return $customOptions;
    }
}
