<?php

/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Cron;

/**
 * Class Cron
 *
 * @package FiloBlu\MappConnector\Cron
 */
class Cron
{

    /**
     * @var \FiloBlu\MappConnector\Helper\FileLoggerHelper
     */
    protected $_fileLogger;

    /**
     * @var \FiloBlu\MappConnector\Model\Config
     */
    protected $_config;

    /**
     * @var \FiloBlu\MappConnector\Console\Command\Prepare
     */
    protected $_prepare;

    /**
     * @var \FiloBlu\MappConnector\Console\Command\SftpProcess
     */
    protected $_sftpProcess;

    /**
     * @var \FiloBlu\MappConnector\Console\Command\RestProcess
     */
    protected $_restProcess;

    /**
     * XML PATHS
     */
    const MAPPCONNECTOR_CRON_REALTIME = "mappconnector/cron/realtime_enabled";
    const MAPPCONNECTOR_CRON_SUBSCRIBER = "mappconnector/cron/subscriber_enabled";
    const MAPPCONNECTOR_CRON_CUSTOMER = "mappconnector/cron/customer_enabled";
    const MAPPCONNECTOR_CRON_PRODUCT = "mappconnector/cron/product_enabled";
    const MAPPCONNECTOR_CRON_ORDER = "mappconnector/cron/order_enabled";
    const MAPPCONNECTOR_CRON_QUOTE = "mappconnector/cron/quote_enabled";
    const MAPPCONNECTOR_SUBSCRIBER_EXPORT_TYPE = "mappconnector/cron/subscriber_export_type";
    const MAPPCONNECTOR_CUSTOMER_EXPORT_TYPE = "mappconnector/cron/customer_export_type";

    /**
     * Cron constructor.
     *
     * @param \FiloBlu\MappConnector\Model\Config            $config
     * @param \FiloBlu\MappConnector\Helper\FileLoggerHelper $fileLogger
     */
    public function __construct(
        \FiloBlu\MappConnector\Model\Config $config,
        \FiloBlu\MappConnector\Helper\FileLoggerHelper $fileLogger,
        \FiloBlu\MappConnector\Console\Command\Prepare $prepare,
        \FiloBlu\MappConnector\Console\Command\SftpProcess $ftpProcess,
        \FiloBlu\MappConnector\Console\Command\RestProcess $restProcess
    ) {
        $this->_fileLogger = $fileLogger;
        $this->_config = $config;
        $this->_prepare = $prepare;
        $this->_sftpProcess = $ftpProcess;
        $this->_restProcess = $restProcess;
    }

    /*
     * PREPARE JOBS SUBSCRIBER
     */
    public function prepareSubscriberIncremental()
    {
        $this->prepareSubscriberIncrementalJobs($this->_config->getConfigValue(self::MAPPCONNECTOR_SUBSCRIBER_EXPORT_TYPE));
    }
    public function prepareSubscriberFullincremental()
    {
        $this->prepareSubscriberFullincrementalJobs($this->_config->getConfigValue(self::MAPPCONNECTOR_SUBSCRIBER_EXPORT_TYPE));
    }

    public function prepareSubscriberIncrementalJobs($type = 'sftp')
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_SUBSCRIBER) == 1) {
            $this->_fileLogger->info('Preparing incremental subscriber ' . $type . ' from Cron Job');
            $this->_prepare->prepareJobs('subscriber', 'incremental', $type);
        } else {
            $this->_fileLogger->info('Cron incremental subscriber ' . $type . ' disabled by config');
        }
    }
    public function prepareSubscriberFullincrementalJobs($type = 'sftp')
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_SUBSCRIBER) == 1) {
            $this->_fileLogger->info('Preparing fullincremental subscriber ' . $type . ' from Cron Job');
            $this->_prepare->prepareJobs('subscriber', 'fullincremental', $type);
        } else {
            $this->_fileLogger->info('Cron fullincremental subscriber ' . $type . ' disabled by config');
        }
    }
    public function prepareSubscriberFullJobs($type = 'sftp')
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_SUBSCRIBER) == 1) {
            $this->_fileLogger->info('Preparing full subscriber ' . $type . ' from Cron Job');
            $this->_prepare->prepareJobs('subscriber', 'full', $type);
        } else {
            $this->_fileLogger->info('Cron full subscriber ' . $type . ' disabled by config');
        }
    }

    /*
     * PREPARE JOBS CUSTOMER
     */
    public function prepareCustomerIncremental()
    {
        $this->prepareCustomerIncrementalJobs($this->_config->getConfigValue(self::MAPPCONNECTOR_CUSTOMER_EXPORT_TYPE));
    }
    public function prepareCustomerFullincremental()
    {
        $this->prepareCustomerFullincrementalJobs($this->_config->getConfigValue(self::MAPPCONNECTOR_CUSTOMER_EXPORT_TYPE));
    }

    public function prepareCustomerIncrementalJobs($type = 'sftp')
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_CUSTOMER) == 1) {
            $this->_fileLogger->info('Preparing incremental customer ' . $type . ' from Cron Job');
            $this->_prepare->prepareJobs('customer', 'incremental', $type);
        } else {
            $this->_fileLogger->info('Cron incremental customer ' . $type . ' disabled by config');
        }
    }
    public function prepareCustomerFullincrementalJobs($type = 'sftp')
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_CUSTOMER) == 1) {
            $this->_fileLogger->info('Preparing fullincremental customer ' . $type . ' from Cron Job');
            $this->_prepare->prepareJobs('customer', 'fullincremental', $type);
        } else {
            $this->_fileLogger->info('Cron fullincremental customer ' . $type . ' disabled by config');
        }
    }
    public function prepareCustomerFullJobs($type = 'sftp')
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_CUSTOMER) == 1) {
            $this->_fileLogger->info('Preparing full customer ' . $type . ' from Cron Job');
            $this->_prepare->prepareJobs('customer', 'full', $type);
        } else {
            $this->_fileLogger->info('Cron full customer ' . $type . ' disabled by config');
        }
    }

    /*
     * PREPARE JOBS QUOTE
     */
    public function prepareQuoteIncremental()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_QUOTE) == 1) {
            $this->_fileLogger->info('Preparing incremental quote sftp from Cron Job');
            $this->_prepare->prepareJobs('quote', 'incremental', 'sftp');
        } else {
            $this->_fileLogger->info('Cron incremental quote sftp disabled by config');
        }
    }
    public function prepareQuoteFullincremental()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_QUOTE) == 1) {
            $this->_fileLogger->info('Preparing fullincremental quote sftp from Cron Job');
            $this->_prepare->prepareJobs('quote', 'fullincremental', 'sftp');
        } else {
            $this->_fileLogger->info('Cron fullincremental quote sftp disabled by config');
        }
    }
    public function prepareQuoteFull()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_QUOTE) == 1) {
            $this->_fileLogger->info('Preparing full quote sftp from Cron Job');
            $this->_prepare->prepareJobs('quote', 'full', 'sftp');
        } else {
            $this->_fileLogger->info('Cron full quote sftp disabled by config');
        }
    }

    /*
     * PREPARE JOBS PRODUCT
     */
    public function prepareProductIncremental()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_PRODUCT) == 1) {
            $this->_fileLogger->info('Preparing incremental product sftp from Cron Job');
            $this->_prepare->prepareJobs('product', 'incremental', 'sftp');
        } else {
            $this->_fileLogger->info('Cron incremental product sftp disabled by config');
        }
    }
    public function prepareProductFullincremental()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_PRODUCT) == 1) {
            $this->_fileLogger->info('Preparing fullincremental product sftp from Cron Job');
            $this->_prepare->prepareJobs('product', 'fullincremental', 'sftp');
        } else {
            $this->_fileLogger->info('Cron fullincremental product sftp disabled by config');
        }
    }
    public function prepareProductFull()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_PRODUCT) == 1) {
            $this->_fileLogger->info('Preparing full product sftp from Cron Job');
            $this->_prepare->prepareJobs('product', 'full', 'sftp');
        } else {
            $this->_fileLogger->info('Cron full product sftp disabled by config');
        }
    }

    /*
     * PREPARE JOBS ORDER
     */
    public function prepareOrderIncremental()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_ORDER) == 1) {
            $this->_fileLogger->info('Preparing incremental order sftp from Cron Job');
            $this->_prepare->prepareJobs('order', 'incremental', 'sftp');
        } else {
            $this->_fileLogger->info('Cron incremental order sftp disabled by config');
        }
    }
    public function prepareOrderFullincremental()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_ORDER) == 1) {
            $this->_fileLogger->info('Preparing fullincremental order sftp from Cron Job');
            $this->_prepare->prepareJobs('order', 'fullincremental', 'sftp');
        } else {
            $this->_fileLogger->info('Cron fullincremental order sftp disabled by config');
        }
    }
    public function prepareOrderFull()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_ORDER) == 1) {
            $this->_fileLogger->info('Preparing full order sftp from Cron Job');
            $this->_prepare->prepareJobs('order', 'full', 'sftp');
        } else {
            $this->_fileLogger->info('Cron full order sftp disabled by config');
        }
    }

    /*
     * PROCESS SFTP JOBS
     */
    public function processSubscriberJob()
    {
        $this->processSubscriberJobType($this->_config->getConfigValue(self::MAPPCONNECTOR_SUBSCRIBER_EXPORT_TYPE));
    }
    public function processSubscriberJobType($type = 'sftp')
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_SUBSCRIBER) == 1) {
            $this->_fileLogger->info('Process waiting subscriber to ' . $type . ' sftp from Cron Job');
            if ($type == 'sftp') {
                $this->_sftpProcess->processSftpJobs('subscriber');
            } elseif ($type == 'rest') {
                $this->_restProcess->processRestJobs('subscriber');
            }
        } else {
            $this->_fileLogger->info('Process waiting subscriber to ' . $type . ' sftp disabled by config');
        }
    }
    public function processCustomerJob()
    {
        $this->processCustomerJobType($this->_config->getConfigValue(self::MAPPCONNECTOR_CUSTOMER_EXPORT_TYPE));
    }
    public function processCustomerJobType($type = 'sftp')
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_CUSTOMER) == 1) {
            $this->_fileLogger->info('Process waiting customer to csv ' . $type . ' from Cron Job');
            if ($type == 'sftp') {
                $this->_sftpProcess->processSftpJobs('customer');
            } elseif ($type == 'rest') {
                $this->_restProcess->processRestJobs('customer');
            }
        } else {
            $this->_fileLogger->info('Process waiting customer to csv ' . $type . ' disabled by config');
        }
    }
    public function processQuoteJob()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_QUOTE) == 1) {
            $this->_fileLogger->info('Process waiting quote to csv sftp from Cron Job');
            $this->_sftpProcess->processSftpJobs('quote');
        } else {
            $this->_fileLogger->info('Process waiting quote to csv ftp disabled by config');
        }
    }
    public function processProductJob()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_PRODUCT) == 1) {
            $this->_fileLogger->info('Process waiting product to csv sftp from Cron Job');
            $this->_sftpProcess->processSftpJobs('product');
        } else {
            $this->_fileLogger->info('Process waiting product to csv sftp disabled by config');
        }
    }
    public function processOrderJob()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_ORDER) == 1) {
            $this->_fileLogger->info('Process waiting order to csv sftp from Cron Job');
            $this->_sftpProcess->processSftpJobs('order');
        } else {
            $this->_fileLogger->info('Process waiting order to csv sftp disabled by config');
        }
    }
    public function processRealtimeJob()
    {
        if ($this->_config->getConfigValue(self::MAPPCONNECTOR_CRON_REALTIME) == 1) {
            $this->_fileLogger->info('Process realtime Jobs');
            $this->_restProcess->processRestJobs();
        } else {
            $this->_fileLogger->info('Process realtime jobs disabled by config');
        }
    }
}
