<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\From\Entity;

use FiloBlu\MappConnector\Model\Config;
use FiloBlu\MappConnector\Model\To\Mapp\FTP;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\UrlInterface;
use Magento\Quote\Model\QuoteFactory;

/**
 * Class QuoteItem
 * @package FiloBlu\MappConnector\Model\From\Entity
 */
class QuoteItem extends AbstractEntity implements EntityInterface
{

    //TODO: Get only abandoned cart

    /**
     * Entity
     *
     * @var string
     */
    protected $_entity = 'quote_item';

    /**
     * Magento Entity Model
     *
     * @var string
     */
    protected $_entityModel = '\Magento\Quote\Model\Quote\Item';

    /**
     * Magento Entity Model Id
     *
     * @var string
     */
    protected $_entityModelPrimary = 'quote_id';

    /**
     * Magento Entity Model Related Id
     *
     * @var string
     */
    protected $_entityModelRelatedPrimary = 'quote_id';

    /**
     * Mapp Resource Model
     *
     * @var string
     */
    protected $_entityMappResource = FTP::class;

    /**
     * Mapp Resource Model Id
     *
     * @var array
     */
    protected $_entityMappPrimary = ['primaryDB' => 'quote_id', 'primaryMapp' => 'email'];

    /**
     * @var string
     */
    protected $_xmlConfigPath = 'mappconnector/quote_mapping/quote_item';

    /**
     * @var Quote
     */
    protected $_quote;

    /**
     * @var QuoteFactory
     */
    protected $_quoteFactory;
    /**
     * @var Config
     */
    protected $_config;

    /**
     * @var ProductRepositoryInterface
     */
    private $productRepository;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var \Magento\Eav\Api\AttributeSetRepositoryInterface
     */
    private $attributeSet;

    /**
     * QuoteItem constructor.
     * @param Quote $quote
     * @param Config $config
     * @param QuoteFactory $quoteFactory
     * @param ProductRepositoryInterface $productRepository
     * @param ResourceConnection $resourceConnection
     * @param \Magento\Eav\Api\AttributeSetRepositoryInterface $attributeSet
     */
    public function __construct(
        Quote $quote,
        Config $config,
        QuoteFactory $quoteFactory,
        ProductRepositoryInterface $productRepository,
        ResourceConnection $resourceConnection,
        \Magento\Eav\Api\AttributeSetRepositoryInterface $attributeSet
    ) {
        $this->_config = $config;
        $this->_quote = $quote;
        $this->_quoteFactory = $quoteFactory;
        $this->productRepository = $productRepository;
        $this->resourceConnection = $resourceConnection;
        $this->attributeSet = $attributeSet;
    }

    /**
     * @param $record
     * @return array @inheritDoc
     * @throws NoSuchEntityException
     */
    public function getDefaultData($record)
    {

        /** @var ProductRepositoryInterface  $repostitory */
        $result = [];

        $q = $this->_quoteFactory->create()->load($record['quote_id']);

        $result['key'] = $q->getCustomerEmail();

        $store = $q->getStore();
        //Get Product
        $product = $this->productRepository->getById($record['product_id']);

        //Get Product Thumbnail
        $thumbnail = ($product->getThumbnail() === 'no_selection') ? '' : $store->getBaseUrl(UrlInterface::URL_TYPE_MEDIA) . 'catalog/product' . $product->getThumbnail();

        //Get Attribute Set Name
        $attributeSetId = $product->getAttributeSetId();
        $attributeSetName = '';
        if (!empty($attributeSetId)) {
            $attributeSet = $this->attributeSet->get($attributeSetId);
            $attributeSetName = $attributeSet->getAttributeSetName();
        }

        // Populate $result with mandatory data
        $result['product_uri'] = $product->getProductUrl();
        $result['product_main_thumbnail'] = $thumbnail;
        $result['product_attribute_set'] = $attributeSetName;

        return $result;
    }

    /**
     * @return {@inheritDoc}
     */
    public function getObjectData($primary, $id)
    {
        $objectManager = ObjectManager::getInstance(); // Instance of object manager
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();
        $tableName = $resource->getTableName('quote_item'); //gives table name with prefix

        $only_simple = $this->_config->getConfigValue('mappconnector/quote_mapping/export_only_simple');

        //Select Data from table
        $sql = "SELECT * FROM {$tableName} WHERE {$primary} = {$id}";

        if ($only_simple == 1) {
            $sql .= ' AND product_type = "simple" ';
        }

        return $connection->fetchAll($sql);
    }

    /**
     * @param $record
     * @return array
     */
    public function getAdditionalData($record)
    {
        $map = $this->_config->getNormalizedConfigObjectByPath('mappconnector/quote_mapping/quote_item_product', 'quote_product');

        $result = $this->getRelatedModelData($this->_quote, $record['quote_id']);

        $p = [];

        try {
            $product = $this->productRepository->getById($record['product_id']);

            foreach ($map as $k => $m) {
                $v = $product->getResource()->getAttribute($k)->getFrontend()->getValue($product);
                if (is_array($v)) {
                    $v = implode(',', $v);
                }
                $p[$m] = (!is_object($v)) ? $v : '';
            }
        } catch (NoSuchEntityException $e) {
            // TODO : log
        }

        return  array_merge($result, $p);
    }

    /**
     * @param string $type
     * @param string $last_index
     * @return mixed
     */
    public function getAllData($type = 'full', $last_index = '')
    {
        $connection = $this->resourceConnection->getConnection();
        $tableName = $this->resourceConnection->getTableName('quote'); //gives table name with prefix
        $tableNameJobs = $this->resourceConnection->getTableName('filoblu_mappconnector_jobs');

        if ($type == 'full') {
            //Full
            //$sql = "Select * FROM " . $tableName;
            $sql = "SELECT count(*)
                    FROM
                    $tableName
                    WHERE
                    is_active = 1
                    AND
                    items_count > 0
                    AND
                    customer_email IS NOT NULL
                    AND
                    TIMESTAMPDIFF(SECOND,created_at,NOW()) > " . $this->_config->getConfigValue('mappconnector/quote_mapping/abandoned_cart_time');

            return $connection->fetchAll($sql);
        }
        //Incremental
        $sql = "SELECT
                        ce.entity_id as entity_id
                    FROM
                        " . $tableName . " AS ce
                    LEFT JOIN
                        " . $tableNameJobs . " AS mj
                    ON
                        mj.entity_id = ce.entity_id
                    AND
                        mj.entity = 'quote'
                    WHERE
                        mj.status is null
                    AND
                        ce.is_active = 1
                    AND
                        ce.items_count > 0
                    AND
                        ce.customer_email IS NOT NULL
                    AND
                        TIMESTAMPDIFF(SECOND,ce.created_at,NOW()) > " . $this->_config->getConfigValue('mappconnector/quote_mapping/abandoned_cart_time');

        return $connection->fetchAll($sql);
    }

    /**
     * @return string
     */
    public function getNormalizedEntityModelPrimary()
    {
        return $this->_quote->getEntityModelPrimary();
    }
}
