<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\To\Mapp\Resource;

use Exception;
use FiloBlu\MappConnector\Adapter\RestAdapter;
use FiloBlu\MappConnector\Helper\ObjectHelper;

/**
 * Class UserResource
 * @package FiloBlu\MappConnector\Model\To\Mapp\Resource
 */
class UserResource extends AbstractResource
{

    /**
     * @var string
     */
    protected $_defaultadapter = RestAdapter::class;

    /**
     * @param $email
     * @param $subscriptionStatus
     * @return array
     * @throws Exception
     */
    public function handleSubscription($email, $subscriptionStatus)
    {
        $subscribe = $forceSubscribe = (bool)$this->_config->getConfigValue('mappconnector/connection/force_subscribe');
        $method = 'unsubscribeByEmail';

        if (!$forceSubscribe) {
            $subscribe = $subscriptionStatus == 1;
        }

        $params = [
                'email' => $email,
                'groupId' => $this->_config->getConfigValue('mappconnector/connection/group_id'),
            ];

        if ($subscribe) {
            $params['subscriptionMode'] = 'OPT_IN';
            $method = 'subscribeByEmail';
        }

        return [
            $this->membership($params, $method),
            "({$method})"
        ];
    }

    /**
     * @param $request
     * @return mixed
     * @throws Exception
     */
    public function doRequest($request)
    {
        //Set the adapter
        //$this->_defaultadapter = $request->config->adapter;

        //If the email domain is excluded return null
        if ($this->_domainIsExcluded($request->request['get']['email'])) {
            return null;
        }

        //Switch request
        switch ($request->config->type) {
            case 'createOrUpdate':
                $get_response = $this->get($request->request['get'], 'getBy' . ucfirst($request->config->mapp_primary['primaryMapp']));

                $user = json_decode($get_response->getResponse()->getBody());
                $gr = $get_response->getResponse()->getBody();
                if (isset($user->id)) {
                    //If exists -> update
                    $create_update_response = $this->save($request->request, 'updateProfileBy' . ucfirst($request->config->mapp_primary['primaryMapp']));
                } else {
                    //If not exists -> create
                    $create_update_response = $this->save($request->request, 'create');
                }

                $cur = $create_update_response->getResponse()->getBody();

                $membership_response = '--';
                $membership_action = '';
                // subscribe/unsubscribe to group id
                if ($this->_config->getConfigValue('mappconnector/connection/use_group_id') == 1 && !empty($this->_config->getConfigValue('mappconnector/connection/group_id'))) {
                    if (isset($request->request['post'][$request->config->map['subscriber_status_normalized']])) {
                        $subscriptionStatus = $request->request['post'][$request->config->map['subscriber_status_normalized']];
                        $emailAddress = $request->request['get']['email'];
                        list($membership_response, $membership_action)  = $this->handleSubscription($emailAddress, $subscriptionStatus);
                    }
                }

                return [
                    'Get User Response',
                    $gr,
                    'Create/Update Response',
                    $cur,
                    'Membership Response ' . $membership_action,
                    ($membership_response != '--') ? $membership_response->getResponse()->getBody() : '--'
                ];

            case 'create':
                return $this->save($request->request, 'create');
            case 'update':
                return $this->save($request->request, 'updateProfileBy' . ucfirst($request->config->mapp_primary['primaryMapp']));
            case 'get':
                return $this->get($request->request['get'], 'getBy' . ucfirst($request->config->mapp_primary['primaryMapp']));
            case 'getProfile':
                return $this->get($request->request['get'], 'getProfileBy' . ucfirst($request->config->mapp_primary['primaryMapp']));
            case 'delete':
                return $this->delete($request->request, 'deleteBy' . ucfirst($request->config->mapp_primary['primaryMapp']));
        }
    }

    /**
     * @param $params
     * @param string $method ['get', 'getByEmail', 'getByIdentifier', 'getByMobileNumber']
     * @return mixed
     * @throws Exception
     */
    public function get($params, $method = 'get')
    {
        $methods = ['get', 'getByEmail', 'getByIdentifier', 'getByMobileNumber', 'getProfile', 'getProfileByEmail', 'getProfileByMobileNumber'];

        if (!in_array($method, $methods)) {
            $this->_fileLogger->error('The method ' . $method . ' for user/get resourse doesn\'t exists', [], __METHOD__ . ':' . __LINE__);
        }

        $this->_data->setMethod('GET');
        $this->_data->setEndpoint('/user/' . $method);

        parent::makeRequest($params);

        return $this->_data;
    }

    /**
     * @param $params
     * @param string $method ['subscribeByEmail', 'unsubscribeByEmail']
     * @return mixed
     * @throws Exception
     */
    public function membership($params, $method = 'post')
    {
        $methods = ['subscribeByEmail', 'unsubscribeByEmail'];

        if (!in_array($method, $methods)) {
            $this->_fileLogger->error('The method ' . $method . ' for user/get resourse doesn\'t exists', [], __METHOD__ . ':' . __LINE__);
        }

        $this->_data->setMethod('POST');
        $this->_data->setEndpoint('/membership/' . $method);

        $p['get'] = $params;

        parent::makeRequest($p);

        return $this->_data;
    }

    /**
     * @param array $params
     * @param string $method
     * @param boolean $jsonPrepare
     * @return mixed
     * @throws Exception
     */
    public function save($params, $method = 'create', $jsonPrepare = true)
    {
        $methods = ['create','updateProfile','updateProfileByEmail','updateProfileByMobileNumber'];

        if (!in_array($method, $methods)) {
            $this->_fileLogger->error('The method ' . $method . ' for user/save resourse doesn\'t exists', [], __METHOD__ . ':' . __LINE__);
        }

        $this->_data->setMethod('POST');
        $this->_data->setEndpoint('/user/' . $method);

        if ($jsonPrepare) {
            $params['post'] = json_encode(ObjectHelper::prepareNameValueObjectArray($params['post']));
        }

        parent::makeRequest($params);

        return $this->_data;
    }

    /**
     * @param array $params
     * @param string $method
     * @return mixed
     * @throws Exception
     */
    public function delete($params, $method = 'delete')
    {
        $methods = ['delete','deleteByEmail'];

        if (!in_array($method, $methods)) {
            $this->_fileLogger->error('The method ' . $method . ' for user/get delete doesn\'t exists', [], __METHOD__ . ':' . __LINE__);
        }

        $this->_data->setMethod('DELETE');
        $this->_data->setEndpoint('/user/' . $method);

        parent::makeRequest($params);

        return $this->_data;
    }

    /**
     * @param $id
     * @return mixed
     * @throws Exception
     */
    public function getUserById($id)
    {
        return $this->get(['userId' => $id]);
    }

    /**
     * @param $email
     * @return mixed
     * @throws Exception
     */
    public function getUserByEmail($email)
    {
        return $this->get(['email' => $email], "getByEmail");
    }

    /**
     * @param $identifier
     * @return mixed
     * @throws Exception
     */
    public function getUserByIdentifier($identifier)
    {
        return $this->get(['identifier' => $identifier], "getByIdentifier");
    }

    /**
     * @param $mobileNumber
     * @return mixed
     * @throws Exception
     */
    public function getUserByMobileNumber($mobileNumber)
    {
        return $this->get(['mobileNumber' => $mobileNumber], "getByMobileNumber");
    }

    /**
     * @param $id
     * @return mixed
     * @throws Exception
     */
    public function getProfileById($id)
    {
        return $this->get(['userId' => $id], 'getProfile');
    }

    /**
     * @param $email
     * @return mixed
     * @throws Exception
     */
    public function getProfileByEmail($email)
    {
        return $this->get(['email' => $email], "getProfileByEmail");
    }

    /**
     * @param $mobileNumber
     * @return mixed
     * @throws Exception
     */
    public function getProfileByMobileNumber($mobileNumber)
    {
        return $this->get(['mobileNumber' => $mobileNumber], "getProfileByMobileNumber");
    }

    /**
     * @param $params
     * @return mixed
     * @throws Exception
     */
    public function create($params)
    {
        return $this->save($params);
    }

    /**
     * @param $params
     * @return mixed
     * @throws Exception
     */
    public function updateProfileById($params)
    {
        $req = ['get' => $params['primary'], 'post' => json_encode(ObjectHelper::prepareNameValueObjectArray($params['info']))];
        return $this->save($req, 'updateProfile');
    }

    /**
     * @param $id
     * @return mixed
     * @throws Exception
     */
    public function deleteById($id)
    {
        return $this->delete(['userId' => $id]);
    }

    /**
     * @param $email
     * @return mixed
     * @throws Exception
     */
    public function deleteByEmail($email)
    {
        return $this->delete(['email' => $email], 'deleteByEmail');
    }
}
